/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { types as t } from '@babel/core';
import convertToPropTypes from './convertBabelToPropTypes';
import extractGenericTypeNames from './extractGenericTypeNames';
import { createPropTypesObject, mergePropTypes } from './propTypes';

function findStaticProperty(node, name) {
  return node.body.body.find(property => t.isClassProperty(property, {
    static: true
  }) && t.isIdentifier(property.key, {
    name
  }) && (t.isObjectExpression(property.value) || t.isCallExpression(property.value)));
}

export default function addToClass(node, state) {
  if (!node.superTypeParameters || node.superTypeParameters.params.length <= 0) {
    return;
  }

  const defaultProps = findStaticProperty(node, 'defaultProps');
  const defaultPropsKeyList = [];

  if (defaultProps && t.isClassProperty(defaultProps) && t.isObjectExpression(defaultProps.value)) {
    defaultProps.value.properties.forEach(prop => {
      if (t.isProperty(prop) && t.isIdentifier(prop.key)) {
        defaultPropsKeyList.push(prop.key.name);
      }
    });
  }

  const typeNames = extractGenericTypeNames(node.superTypeParameters.params[0]);
  const propTypesList = convertToPropTypes(state.componentTypes, typeNames, state, defaultPropsKeyList);

  if (typeNames.length === 0 || propTypesList.length === 0) {
    return;
  }

  const propTypes = findStaticProperty(node, 'propTypes');

  if (propTypes) {
    propTypes.value = mergePropTypes(propTypes.value, propTypesList, state);
  } else {
    const staticProperty = t.classProperty(t.identifier('propTypes'), createPropTypesObject(propTypesList, state));
    staticProperty.static = true;
    node.body.body.unshift(staticProperty);
  }
}