/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { types as t } from '@babel/core';
export default function extractTypeProperties(node, types) {
  const properties = [];

  const mapToPropertySignature = data => {
    data.forEach(prop => {
      if (t.isTSPropertySignature(prop)) {
        properties.push(prop);
      }
    });
  };

  if (t.isIdentifier(node)) {
    if (types[node.name]) {
      properties.push(...types[node.name]);
    }
  } else if (t.isTSTypeReference(node)) {
    properties.push(...extractTypeProperties(node.typeName, types));
  } else if (t.isTSInterfaceDeclaration(node)) {
    (node.extends || []).forEach(ext => {
      properties.push(...extractTypeProperties(ext.expression, types));
    });
    mapToPropertySignature(node.body.body);
  } else if (t.isTSTypeAliasDeclaration(node)) {
    properties.push(...extractTypeProperties(node.typeAnnotation, types));
  } else if (t.isTSTypeLiteral(node)) {
    mapToPropertySignature(node.members);
  } else if (t.isTSIntersectionType(node) || t.isTSUnionType(node)) {
    node.types.forEach(intType => {
      properties.push(...extractTypeProperties(intType, types));
    });
  }

  return properties;
}