/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { types as t } from '@babel/core';
export function hasCustomPropTypeSuffix(name, suffixes) {
  return !!suffixes && suffixes.some(suffix => name.endsWith(suffix));
}
export function isReactTypeMatch(name, type, reactImportedName) {
  return name === type || name === `React.${type}` || name === `${reactImportedName}.${type}`;
}
export function wrapIsRequired(propType, optional) {
  return optional ? propType : t.memberExpression(propType, t.identifier('isRequired'));
}
export function createMember(value, propTypesImportedName) {
  return t.memberExpression(t.identifier(propTypesImportedName), value);
}
export function createCall(value, args, propTypesImportedName) {
  return t.callExpression(createMember(value, propTypesImportedName), args);
}
export function createPropTypesObject(propTypes, state) {
  const object = t.objectExpression(propTypes);
  return state.options.forbidExtraProps ? t.callExpression(t.identifier(state.airbnbPropTypes.forbidImport), [object]) : object;
}
export function mergePropTypes(expr, propTypes, state, wrapForbid = true) {
  if (t.isCallExpression(expr)) {
    if (t.isIdentifier(expr.callee, {
      name: 'forbidExtraProps'
    })) {
      expr.arguments.forEach((arg, index) => {
        expr.arguments[index] = mergePropTypes(arg, propTypes, state, false);
      });
    }

    return expr;
  }

  if (!t.isObjectExpression(expr)) {
    return expr;
  }

  const {
    properties
  } = expr;
  const existingProps = {};
  properties.forEach(property => {
    if (t.isObjectProperty(property) && t.isIdentifier(property.key)) {
      existingProps[property.key.name] = true;
    }
  });
  propTypes.forEach(propType => {
    if (t.isIdentifier(propType.key) && !existingProps[propType.key.name]) {
      properties.unshift(propType);
    }
  });

  if (wrapForbid && state.options.forbidExtraProps) {
    return t.callExpression(t.identifier(state.airbnbPropTypes.forbidImport), [expr]);
  }

  return expr;
}