/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
import { types as t } from '@babel/core';
export default function upsertImport(imp, options = {}) {
  const {
    checkForDefault,
    checkForNamed
  } = options;
  const response = {
    defaultImport: '',
    namedImport: '',
    namedImports: []
  };
  let hasDefault = false;
  let hasNamed = false;
  response.namedImports = imp.specifiers.filter(spec => {
    if (t.isImportDefaultSpecifier(spec) || t.isImportNamespaceSpecifier(spec)) {
      response.defaultImport = spec.local.name;
      hasDefault = true;
      return false;
    }

    return true;
  }).map(spec => {
    const {
      name
    } = spec.local;

    if (name === checkForNamed) {
      response.namedImport = name;
      hasNamed = true;
    }

    return name;
  });

  if (checkForDefault && !hasDefault) {
    imp.specifiers.unshift(t.importDefaultSpecifier(t.identifier(checkForDefault)));
    response.defaultImport = checkForDefault;
  }

  if (checkForNamed && !hasNamed) {
    imp.specifiers.push(t.importSpecifier(t.identifier(checkForNamed), t.identifier(checkForNamed)));
    response.namedImport = checkForNamed;
  }

  return response;
}