"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = addToClass;

var _core = require("@babel/core");

var _convertBabelToPropTypes = _interopRequireDefault(require("./convertBabelToPropTypes"));

var _extractGenericTypeNames = _interopRequireDefault(require("./extractGenericTypeNames"));

var _propTypes = require("./propTypes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
function findStaticProperty(node, name) {
  return node.body.body.find(property => _core.types.isClassProperty(property, {
    static: true
  }) && _core.types.isIdentifier(property.key, {
    name
  }) && (_core.types.isObjectExpression(property.value) || _core.types.isCallExpression(property.value)));
}

function addToClass(node, state) {
  if (!node.superTypeParameters || node.superTypeParameters.params.length <= 0) {
    return;
  }

  const defaultProps = findStaticProperty(node, 'defaultProps');
  const defaultPropsKeyList = [];

  if (defaultProps && _core.types.isClassProperty(defaultProps) && _core.types.isObjectExpression(defaultProps.value)) {
    defaultProps.value.properties.forEach(prop => {
      if (_core.types.isProperty(prop) && _core.types.isIdentifier(prop.key)) {
        defaultPropsKeyList.push(prop.key.name);
      }
    });
  }

  const typeNames = (0, _extractGenericTypeNames.default)(node.superTypeParameters.params[0]);
  const propTypesList = (0, _convertBabelToPropTypes.default)(state.componentTypes, typeNames, state, defaultPropsKeyList);

  if (typeNames.length === 0 || propTypesList.length === 0) {
    return;
  }

  const propTypes = findStaticProperty(node, 'propTypes');

  if (propTypes) {
    propTypes.value = (0, _propTypes.mergePropTypes)(propTypes.value, propTypesList, state);
  } else {
    const staticProperty = _core.types.classProperty(_core.types.identifier('propTypes'), (0, _propTypes.createPropTypesObject)(propTypesList, state));

    staticProperty.static = true;
    node.body.body.unshift(staticProperty);
  }
}