"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = addToFunctionOrVar;

var _core = require("@babel/core");

var _convertBabelToPropTypes = _interopRequireDefault(require("./convertBabelToPropTypes"));

var _extractGenericTypeNames = _interopRequireDefault(require("./extractGenericTypeNames"));

var _propTypes = require("./propTypes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
function extractTypeNames(path) {
  if (_core.types.isFunctionDeclaration(path.node)) {
    return (0, _extractGenericTypeNames.default)(path.node.params[0].typeAnnotation.typeAnnotation);
  }

  if (_core.types.isVariableDeclaration(path.node)) {
    return (0, _extractGenericTypeNames.default)(path.node.declarations[0].id.typeAnnotation.typeAnnotation.typeParameters.params[0]);
  }

  return [];
}

function findStaticProperty(path, funcName, name) {
  const expr = path.getAllNextSiblings().find(sibPath => _core.types.isExpressionStatement(sibPath.node) && _core.types.isAssignmentExpression(sibPath.node.expression, {
    operator: '='
  }) && _core.types.isMemberExpression(sibPath.node.expression.left) && _core.types.isObjectExpression(sibPath.node.expression.right) && _core.types.isIdentifier(sibPath.node.expression.left.object, {
    name: funcName
  }) && _core.types.isIdentifier(sibPath.node.expression.left.property, {
    name
  }));
  return expr && expr.node.expression;
}

function addToFunctionOrVar(path, name, state) {
  const rootPath = _core.types.isExportNamedDeclaration(path.parent) || _core.types.isExportDefaultDeclaration(path.parent) ? path.parentPath : path;
  const defaultProps = findStaticProperty(rootPath, name, 'defaultProps');
  const defaultPropsKeyList = [];

  if (defaultProps && _core.types.isAssignmentExpression(defaultProps) && _core.types.isObjectExpression(defaultProps.right)) {
    defaultProps.right.properties.forEach(prop => {
      if (_core.types.isProperty(prop) && _core.types.isIdentifier(prop.key)) {
        defaultPropsKeyList.push(prop.key.name);
      }
    });
  }

  const typeNames = extractTypeNames(path);
  const propTypesList = (0, _convertBabelToPropTypes.default)(state.componentTypes, typeNames, state, defaultPropsKeyList);

  if (typeNames.length === 0 || propTypesList.length === 0) {
    return;
  }

  const propTypes = findStaticProperty(rootPath, name, 'propTypes');

  if (propTypes) {
    propTypes.right = (0, _propTypes.mergePropTypes)(propTypes.right, propTypesList, state);
  } else {
    rootPath.insertAfter(_core.types.expressionStatement(_core.types.assignmentExpression('=', _core.types.memberExpression(_core.types.identifier(name), _core.types.identifier('propTypes')), (0, _propTypes.createPropTypesObject)(propTypesList, state))));
  }
}