"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.convert = convert;
exports.convertSymbolFromSource = convertSymbolFromSource;

var _core = require("@babel/core");

var _typescript = _interopRequireDefault(require("typescript"));

var _propTypes = require("./propTypes");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
function convert(type, state, depth) {
  const {
    reactImportedName,
    propTypes
  } = state;
  const propTypesImportedName = propTypes.defaultImport;
  const isMaxDepth = depth >= (state.options.maxDepth || 3);
  state.propTypes.count += 1;

  if (type.flags & _typescript.default.TypeFlags.Any) {
    return (0, _propTypes.createMember)(_core.types.identifier('any'), propTypesImportedName);
  } else if (type.flags & _typescript.default.TypeFlags.StringLike) {
    return (0, _propTypes.createMember)(_core.types.identifier('string'), propTypesImportedName);
  } else if (type.flags & _typescript.default.TypeFlags.NumberLike) {
    return (0, _propTypes.createMember)(_core.types.identifier('number'), propTypesImportedName);
  } else if (type.flags & _typescript.default.TypeFlags.BooleanLike) {
    return (0, _propTypes.createMember)(_core.types.identifier('bool'), propTypesImportedName);
  } else if (type.flags & _typescript.default.TypeFlags.ESSymbolLike) {
    return (0, _propTypes.createMember)(_core.types.identifier('symbol'), propTypesImportedName);
  } else if (type.flags & _typescript.default.TypeFlags.Object) {
    const objType = type;

    if (objType.objectFlags & _typescript.default.ObjectFlags.Interface) {} else if (objType.objectFlags & _typescript.default.ObjectFlags.Reference) {} else if (objType.objectFlags & _typescript.default.ObjectFlags.Tuple) {} else if (objType.objectFlags & _typescript.default.ObjectFlags.Mapped) {} else if (objType.objectFlags & _typescript.default.ObjectFlags.Instantiated) {} else if (objType.objectFlags & _typescript.default.ObjectFlags.ObjectLiteral) {}

    return (0, _propTypes.createMember)(_core.types.identifier('object'), propTypesImportedName);
  } else if (type.flags & _typescript.default.TypeFlags.Any) {
    return (0, _propTypes.createMember)(_core.types.identifier('func'), propTypesImportedName);
  } else if (_core.types.isTSTypeReference(type)) {
    const name = '';

    if ((0, _propTypes.isReactTypeMatch)(name, 'ReactText', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'ReactNode', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'ReactType', reactImportedName)) {
      return (0, _propTypes.createMember)(_core.types.identifier('node'), propTypesImportedName);
    } else if ((0, _propTypes.isReactTypeMatch)(name, 'ComponentType', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'ComponentClass', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'StatelessComponent', reactImportedName)) {
      return (0, _propTypes.createMember)(_core.types.identifier('func'), propTypesImportedName);
    } else if ((0, _propTypes.isReactTypeMatch)(name, 'Element', 'JSX') || (0, _propTypes.isReactTypeMatch)(name, 'ReactElement', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'ComponentElement', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'FunctionComponentElement', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'DOMElement', reactImportedName) || (0, _propTypes.isReactTypeMatch)(name, 'SFCElement', reactImportedName)) {
      return (0, _propTypes.createMember)(_core.types.identifier('element'), propTypesImportedName);
    } else if ((0, _propTypes.isReactTypeMatch)(name, 'Ref', reactImportedName)) {
      return (0, _propTypes.createCall)(_core.types.identifier('oneOfType'), [_core.types.arrayExpression([(0, _propTypes.createMember)(_core.types.identifier('string'), propTypesImportedName), (0, _propTypes.createMember)(_core.types.identifier('func'), propTypesImportedName), (0, _propTypes.createMember)(_core.types.identifier('object'), propTypesImportedName)])], propTypesImportedName);
    } else if (name.endsWith('Handler')) {
      return (0, _propTypes.createMember)(_core.types.identifier('func'), propTypesImportedName);
    } else if (name.endsWith('Event')) {
      return (0, _propTypes.createMember)(_core.types.identifier('object'), propTypesImportedName);
    } else if ((0, _propTypes.hasCustomPropTypeSuffix)(name, state.options.customPropTypeSuffixes)) {
      return _core.types.identifier(name);
    } else if (state.typeChecker) {
      return convertSymbolFromSource(state.filePath, name, state);
    }

    return (0, _propTypes.createMember)(_core.types.identifier('any'), propTypesImportedName);
  } else if (_core.types.isTSArrayType(type)) {} else if (_core.types.isTSTypeLiteral(type)) {
    if (type.members.length === 0 || isMaxDepth) {
      return (0, _propTypes.createMember)(_core.types.identifier('object'), propTypesImportedName);
    } else if (type.members.length === 1 && _core.types.isTSIndexSignature(type.members[0])) {} else {}
  } else if (_core.types.isTSUnionType(type) || _core.types.isTSIntersectionType(type)) {} else if (_core.types.isTSInterfaceDeclaration(type)) {} else if (_core.types.isTSTypeAliasDeclaration(type)) {}

  state.propTypes.count -= 1;
  return null;
}

function convertSymbolFromSource(filePath, symbolName, state) {
  const program = state.typeProgram;
  const checker = state.typeChecker;
  const source = program.getSourceFile(filePath);

  if (!source) {
    return null;
  }

  let refNode = null;
  source.statements.some(node => {
    if (_typescript.default.isImportDeclaration(node) && node.importClause) {
      const {
        name,
        namedBindings
      } = node.importClause;

      if (name && name.text === symbolName) {
        refNode = name;
        return true;
      }

      if (!refNode && namedBindings && _typescript.default.isNamedImports(namedBindings)) {
        return namedBindings.elements.some(element => {
          if (element.name.text === symbolName) {
            refNode = element.name;
            return true;
          }

          return false;
        });
      }
    }

    return false;
  });

  if (!refNode) {
    return null;
  }

  const symbol = checker.getSymbolAtLocation(refNode);

  if (!symbol) {
    return null;
  }

  return null;
}