"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = extractTypeProperties;

var _core = require("@babel/core");

/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
function extractTypeProperties(node, types) {
  const properties = [];

  const mapToPropertySignature = data => {
    data.forEach(prop => {
      if (_core.types.isTSPropertySignature(prop)) {
        properties.push(prop);
      }
    });
  };

  if (_core.types.isIdentifier(node)) {
    if (types[node.name]) {
      properties.push(...types[node.name]);
    }
  } else if (_core.types.isTSTypeReference(node)) {
    properties.push(...extractTypeProperties(node.typeName, types));
  } else if (_core.types.isTSInterfaceDeclaration(node)) {
    (node.extends || []).forEach(ext => {
      properties.push(...extractTypeProperties(ext.expression, types));
    });
    mapToPropertySignature(node.body.body);
  } else if (_core.types.isTSTypeAliasDeclaration(node)) {
    properties.push(...extractTypeProperties(node.typeAnnotation, types));
  } else if (_core.types.isTSTypeLiteral(node)) {
    mapToPropertySignature(node.members);
  } else if (_core.types.isTSIntersectionType(node) || _core.types.isTSUnionType(node)) {
    node.types.forEach(intType => {
      properties.push(...extractTypeProperties(intType, types));
    });
  }

  return properties;
}