"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.hasCustomPropTypeSuffix = hasCustomPropTypeSuffix;
exports.isReactTypeMatch = isReactTypeMatch;
exports.wrapIsRequired = wrapIsRequired;
exports.createMember = createMember;
exports.createCall = createCall;
exports.createPropTypesObject = createPropTypesObject;
exports.mergePropTypes = mergePropTypes;

var _core = require("@babel/core");

/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
function hasCustomPropTypeSuffix(name, suffixes) {
  return !!suffixes && suffixes.some(suffix => name.endsWith(suffix));
}

function isReactTypeMatch(name, type, reactImportedName) {
  return name === type || name === `React.${type}` || name === `${reactImportedName}.${type}`;
}

function wrapIsRequired(propType, optional) {
  return optional ? propType : _core.types.memberExpression(propType, _core.types.identifier('isRequired'));
}

function createMember(value, propTypesImportedName) {
  return _core.types.memberExpression(_core.types.identifier(propTypesImportedName), value);
}

function createCall(value, args, propTypesImportedName) {
  return _core.types.callExpression(createMember(value, propTypesImportedName), args);
}

function createPropTypesObject(propTypes, state) {
  const object = _core.types.objectExpression(propTypes);

  return state.options.forbidExtraProps ? _core.types.callExpression(_core.types.identifier(state.airbnbPropTypes.forbidImport), [object]) : object;
}

function mergePropTypes(expr, propTypes, state, wrapForbid = true) {
  if (_core.types.isCallExpression(expr)) {
    if (_core.types.isIdentifier(expr.callee, {
      name: 'forbidExtraProps'
    })) {
      expr.arguments.forEach((arg, index) => {
        expr.arguments[index] = mergePropTypes(arg, propTypes, state, false);
      });
    }

    return expr;
  }

  if (!_core.types.isObjectExpression(expr)) {
    return expr;
  }

  const {
    properties
  } = expr;
  const existingProps = {};
  properties.forEach(property => {
    if (_core.types.isObjectProperty(property) && _core.types.isIdentifier(property.key)) {
      existingProps[property.key.name] = true;
    }
  });
  propTypes.forEach(propType => {
    if (_core.types.isIdentifier(propType.key) && !existingProps[propType.key.name]) {
      properties.unshift(propType);
    }
  });

  if (wrapForbid && state.options.forbidExtraProps) {
    return _core.types.callExpression(_core.types.identifier(state.airbnbPropTypes.forbidImport), [expr]);
  }

  return expr;
}