"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.loadTSConfig = loadTSConfig;
exports.loadProgram = loadProgram;

var _fs = _interopRequireDefault(require("fs"));

var _path = _interopRequireDefault(require("path"));

var _fastGlob = _interopRequireDefault(require("fast-glob"));

var _typescript = _interopRequireDefault(require("typescript"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
let config;
let program;

function loadTSConfig() {
  if (config) {
    return config;
  }

  const {
    config: maybeConfig,
    error
  } = _typescript.default.readConfigFile(_path.default.join(process.cwd(), 'tsconfig.json'), filePath => _fs.default.readFileSync(filePath, 'utf8'));

  if (error) {
    throw error;
  }

  const {
    options,
    errors
  } = _typescript.default.parseJsonConfigFileContent(maybeConfig, _typescript.default.sys, process.cwd());

  if (errors.length > 0) {
    throw errors[0];
  }

  config = options;
  return options;
}

function loadProgram(pattern, root) {
  if (program) {
    return program;
  }

  program = _typescript.default.createProgram(_fastGlob.default.sync(pattern === true ? './src/**/*.ts' : pattern, {
    absolute: true,
    cwd: root
  }), loadTSConfig());
  return program;
}