"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = upsertImport;

var _core = require("@babel/core");

/**
 * @copyright   2018-2019, Miles Johnson
 * @license     https://opensource.org/licenses/MIT
 */
function upsertImport(imp, options = {}) {
  const {
    checkForDefault,
    checkForNamed
  } = options;
  const response = {
    defaultImport: '',
    namedImport: '',
    namedImports: []
  };
  let hasDefault = false;
  let hasNamed = false;
  response.namedImports = imp.specifiers.filter(spec => {
    if (_core.types.isImportDefaultSpecifier(spec) || _core.types.isImportNamespaceSpecifier(spec)) {
      response.defaultImport = spec.local.name;
      hasDefault = true;
      return false;
    }

    return true;
  }).map(spec => {
    const {
      name
    } = spec.local;

    if (name === checkForNamed) {
      response.namedImport = name;
      hasNamed = true;
    }

    return name;
  });

  if (checkForDefault && !hasDefault) {
    imp.specifiers.unshift(_core.types.importDefaultSpecifier(_core.types.identifier(checkForDefault)));
    response.defaultImport = checkForDefault;
  }

  if (checkForNamed && !hasNamed) {
    imp.specifiers.push(_core.types.importSpecifier(_core.types.identifier(checkForNamed), _core.types.identifier(checkForNamed)));
    response.namedImport = checkForNamed;
  }

  return response;
}